using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace Toolkit.Threading {
	/// <summary>
	/// OoƂ̃Cxg̈
	/// </summary>
	public class UnhandledExceptionEventArgs : EventArgs {
		Exception exception;

		public UnhandledExceptionEventArgs(Exception exception) {
			this.exception = exception;
		}

		/// <summary>
		/// ̗OIuWFNg
		/// </summary>
		public Exception Exception { get { return exception; } }
	}

	/// <summary>
	/// dȃNXB
	/// </summary>
	public interface IWorkerThreadWork : IEquatable<IWorkerThreadWork> {
		/// <summary>
		/// d̎s
		/// </summary>
		/// <param name="sender">̃\bhĂяoIuWFNg</param>
		void DoWork(WorkerThread sender);
	}

	/// <summary>
	/// ʃXbhłds߂̃NXB
	/// </summary>
	public class WorkerThread : IDisposable {
		Thread thread;
		bool threadValid;
		List<WorkInfo> works = new List<WorkInfo>();
		IWorkerThreadWork executingWork = null;

		public WorkerThread() {
			threadValid = true;
			thread = new Thread(new ThreadStart(OnThread));
			thread.Start();
		}

		/// <summary>
		/// nB
		/// </summary>
		public void Dispose() {
			System.Diagnostics.Debug.Assert(threadValid);
			lock (works) {
				threadValid = false;
				works.Clear();
				Monitor.Pulse(works);
			}
			thread.Join();
		}

		/// <summary>
		/// Ȍ
		/// </summary>
		public event EventHandler<UnhandledExceptionEventArgs> UnhandledException;

		/// <summary>
		/// d̒ǉB
		/// </summary>
		/// <param name="work">d</param>
		/// <exception cref="ObjectDisposedException">Dispose()Ă΂ꂽɌĂ΂ꂽ</exception>
		public void Add(IWorkerThreadWork work) {
			Add(work, DateTime.Now);
		}

		/// <summary>
		/// d̒ǉB
		/// </summary>
		/// <param name="work">d</param>
		/// <param name="time">ds鎞ԁBȗDateTime.Now</param>
		/// <exception cref="ObjectDisposedException">Dispose()Ă΂ꂽɌĂ΂ꂽ</exception>
		public void Add(IWorkerThreadWork work, DateTime time) {
			lock (works) {
				if (!threadValid) {
					throw new ObjectDisposedException(GetType().ToString());
				}
				// \[gȒǉ
				for (int i = 0; ; i++) {
					if (works.Count <= i) {
						works.Add(new WorkInfo(time, work));
						break;
					} else if (time < works[i].DateTime) {
						works.Insert(i, new WorkInfo(time, work));
						break;
					}
				}
				Monitor.Pulse(works);
			}
		}

		/// <summary>
		/// s҂̃Xgwork܂܂ĂtrueԂB
		/// </summary>
		/// <param name="work">d</param>
		public bool Contains(IWorkerThreadWork work) {
			lock (works) {
				return works.Exists(delegate(WorkInfo wi) {
					return work.Equals(wi.Work);
				});
			}
		}

		/// <summary>
		/// s҂̃XgAworkƓ̂폜B
		/// </summary>
		/// <param name="work">d</param>
		/// <returns>폜</returns>
		public int Remove(IWorkerThreadWork work) {
			lock (works) {
				return works.RemoveAll(delegate(WorkInfo wi) {
					return work.Equals(wi.Work);
				});
			}
		}

		/// <summary>
		/// s҂̃XgÂ폜B
		/// </summary>
		/// <param name="pr">폜vf̏</param>
		/// <returns>폜</returns>
		public int RemoveAll(Predicate<IWorkerThreadWork> pr) {
			lock (works) {
				return works.RemoveAll(delegate(WorkInfo wi) {
					return pr(wi.Work);
				});
			}
		}

		/// <summary>
		/// s҂̂dj
		/// </summary>
		public void Clear() {
			lock (works) {
				works.Clear();
			}
		}

		/// <summary>
		/// ܂ĂWork̐B
		/// </summary>
		public int Count {
			get { return works.Count; }
		}

		/// <summary>
		/// Xbh̎擾
		/// </summary>
		public Thread Thread {
			get { return thread; }
		}

		/// <summary>
		/// ŝd擾BsĂȂnull
		/// </summary>
		public IWorkerThreadWork ExecutingWork {
			get { return executingWork; }
		}

		/// <summary>
		/// XbhB
		/// </summary>
		void OnThread() {
			try {
				while (threadValid) {
					WorkInfo work;
					lock (works) {
						work = PeekWork();
						if (work == null) {
							Monitor.Wait(works);
							continue;
						} else {
							DateTime now = DateTime.Now;
							if (now < work.DateTime) {
								Monitor.Wait(works, work.DateTime - now);
								continue;
							}
						}
						works.Remove(work);
					}
					try {
						executingWork = work.Work;
						work.Work.DoWork(this);
					} catch (Exception e) {
						EventHandler<UnhandledExceptionEventArgs> UnhandledException = this.UnhandledException;
						if (UnhandledException != null) {
							UnhandledException(this, new UnhandledExceptionEventArgs(e));
						} else {
							System.Diagnostics.Debug.Fail(e.Message);
						}
					} finally {
						executingWork = null;
					}
				}
			} catch (Exception e) {
				if (UnhandledException != null) {
					UnhandledException(this, new UnhandledExceptionEventArgs(e));
				} else {
					System.Diagnostics.Debug.Fail(e.Message);
				}
			}
		}

		private WorkInfo PeekWork() {
			if (works.Count <= 0) {
				return null;
			}
			return works[0];
		}

		class WorkInfo {
			public DateTime DateTime;
			public IWorkerThreadWork Work;
			public WorkInfo(DateTime n, IWorkerThreadWork w) {
				DateTime = n;
				Work = w;
			}
		}
	}
}
