﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ふせん {
	public class Config : Toolkit.Serializable, ICloneable {
		public string FontFamily = "ＭＳ Ｐゴシック";
		public float FontSize = 24f;
		public FontStyle FontStyle = FontStyle.Regular;
		public ARGB FontColor = new ARGB(Color.White);
		public bool AntiAlias = true;
		public int DecorationType = 2;
		public ARGB DecorationColor = new ARGB(Color.Gray);
		public ARGB TagColor = new ARGB(Color.Black);
		public Size TagMargin = new Size(32, 24);
		public int Opacity = 100;
		public bool Snap = true;

		public int EditorType = 1;
		public string EditorPath = "%windir%\\notepad.exe";
		public int DoubleClickAction = 1;

		public Toolkit.Updater.Data.UpdateConfig UpdateConfig = new Toolkit.Updater.Data.UpdateConfig();

		#region Serialize/Deserialize

		/// <summary>
		/// 書き込み
		/// </summary>
		public void Serialize() {
			Serialize(".cfg");
		}

		/// <summary>
		/// 読み込み
		/// </summary>
		public static Config Deserialize() {
			try {
				return Deserialize<Config>(".cfg");
			} catch {
				return new Config();
			}
		}

		#endregion

		/// <summary>
		/// 複製の作成
		/// </summary>
		public Config Clone() {
			Config copy = (Config)MemberwiseClone();
			// string以外の参照型なメンバがあればここでコピー
			return copy;
		}

		#region ICloneable メンバ

		object ICloneable.Clone() {
			return Clone();
		}

		#endregion
	}

	/// <summary>
	/// Colorが何故かうまくシリアライズ出来ないので。。
	/// </summary>
	[Serializable]
	public struct ARGB {
		public byte A;
		public byte R;
		public byte G;
		public byte B;

		public ARGB(Color c) {
			A = c.A;
			R = c.R;
			G = c.G;
			B = c.B;
		}

		public static explicit operator Color(ARGB b) {
			return Color.FromArgb(b.A, b.R, b.G, b.B);
		}
	}
}
