﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics; // Processとか

namespace ふせん
{
	public partial class ConfigForm : Form
	{
		Config config;
		TagDrawer drawer;
		bool initialized = false;

		public Config Config { get { return config; } }

		public ConfigForm(Config config) {
			InitializeComponent();

			this.config = config.Clone();
			drawer = new TagDrawer(this.config);

			try {
				pictureBox1.Image = new Bitmap(pictureBox1.ClientSize.Width, pictureBox1.ClientSize.Height);

				switch (config.DecorationType) {
				case 0: radioButton7.Checked = true; break;
				case 1: radioButton8.Checked = true; break;
				case 2: radioButton9.Checked = true; break;
				}
				checkBox2.Checked = config.AntiAlias;
				numericUpDown2.Value = (decimal)config.TagMargin.Height;
				numericUpDown3.Value = (decimal)config.TagMargin.Width;
				numericUpDown1.Value = (decimal)config.Opacity;
				checkBox1.Checked = config.Snap;
				switch (config.EditorType) {
				case 0: radioButton1.Checked = true; break;
				case 1: radioButton2.Checked = true; break;
				case 2: radioButton3.Checked = true; break;
				}
				textBox1.Text = config.EditorPath;
				switch (config.DoubleClickAction) {
				case 0: radioButton4.Checked = true; break;
				case 1: radioButton5.Checked = true; break;
				case 2: radioButton6.Checked = true; break;
				}
			} finally {
				initialized = true;
			}
			UpdateSample();
		}

		/// <summary>
		/// OKボタン押された
		/// </summary>
		private void buttonOk_Click(object sender, EventArgs e) {
			SaveToConfig();
		}

		/// <summary>
		/// configに設定を保存
		/// </summary>
		private void SaveToConfig() {
			if (radioButton7.Checked) config.DecorationType = 0;
			if (radioButton8.Checked) config.DecorationType = 1;
			if (radioButton9.Checked) config.DecorationType = 2;
			config.AntiAlias = checkBox2.Checked;
			config.TagMargin.Height = (int)numericUpDown2.Value;
			config.TagMargin.Width = (int)numericUpDown3.Value;
			config.Opacity = (int)numericUpDown1.Value;
			config.Snap = checkBox1.Checked;
			if (radioButton1.Checked) config.EditorType = 0;
			if (radioButton2.Checked) config.EditorType = 1;
			if (radioButton3.Checked) config.EditorType = 2;
			config.EditorPath = textBox1.Text;
			if (radioButton4.Checked) config.DoubleClickAction = 0;
			if (radioButton5.Checked) config.DoubleClickAction = 1;
			if (radioButton6.Checked) config.DoubleClickAction = 2;
		}

		/// <summary>
		/// 後始末
		/// </summary>
		private void ConfigForm_FormClosing(object sender, FormClosingEventArgs e) {
			drawer.Dispose();
		}

		#region 付箋な辺り

		/// <summary>
		/// フォント設定ボタン
		/// </summary>
		private void button1_Click(object sender, EventArgs e) {
			fontDialog1.Font = drawer.Font;
			fontDialog1.Color = (Color)config.FontColor;
			if (fontDialog1.ShowDialog(this) == DialogResult.OK) {
				config.FontFamily = fontDialog1.Font.FontFamily.Name;
				config.FontSize = fontDialog1.Font.Size;
				config.FontStyle = fontDialog1.Font.Style;
				config.FontColor = new ARGB(fontDialog1.Color);
				OnChangeConfig();
				// 念のため再設定
				fontDialog1.Font = drawer.Font;
			}
		}

		/// <summary>
		/// 背景色設定ボタン
		/// </summary>
		private void button3_Click(object sender, EventArgs e) {
			colorDialog1.Color = (Color)config.TagColor;
			if (colorDialog1.ShowDialog(this) == DialogResult.OK) {
				config.TagColor = new ARGB(colorDialog1.Color);
				OnChangeConfig();
			}
		}

		/// <summary>
		/// 装飾色設定ボタン
		/// </summary>
		private void button8_Click(object sender, EventArgs e) {
			colorDialog1.Color = (Color)config.DecorationColor;
			if (colorDialog1.ShowDialog(this) == DialogResult.OK) {
				config.DecorationColor = new ARGB(colorDialog1.Color);
				OnChangeConfig();
			}
		}

		private void radioButton7_CheckedChanged(object sender, EventArgs e) {
			OnChangeConfig();
		}

		private void radioButton8_CheckedChanged(object sender, EventArgs e) {
			OnChangeConfig();
		}

		private void radioButton9_CheckedChanged(object sender, EventArgs e) {
			OnChangeConfig();
		}

		private void checkBox2_CheckedChanged(object sender, EventArgs e) {
			OnChangeConfig();
		}

		private void numericUpDown2_ValueChanged(object sender, EventArgs e) {
			OnChangeConfig();
		}

		private void numericUpDown3_ValueChanged(object sender, EventArgs e) {
			OnChangeConfig();
		}

		/// <summary>
		/// コンフィグ変更されたときの処理
		/// </summary>
		private void OnChangeConfig() {
			if (initialized) { // 初期化済みな場合のみ処理
				SaveToConfig();
				drawer.Dispose();
				drawer = new TagDrawer(config);
				UpdateSample();
			}
		}
		/// <summary>
		/// サンプル表示の更新
		/// </summary>
		private void UpdateSample() {
			using (Graphics g = Graphics.FromImage(pictureBox1.Image)) {
				using (SolidBrush br = new SolidBrush(SystemColors.Control)) {
					g.FillRectangle(br, pictureBox1.ClientRectangle);
				}
				string txt = "ABCdef\r\nあいう";
				Size sz = drawer.GetSize(g, txt);
				Rectangle rcText = new Rectangle(
					new Point((pictureBox1.ClientSize.Width - sz.Width) / 2,
							  (pictureBox1.ClientSize.Height - sz.Height) / 2),
					sz);
				rcText.Intersect(pictureBox1.ClientRectangle);
				drawer.Draw(g, txt, rcText);
			}
			pictureBox1.Invalidate();
			pictureBox1.Update();
		}

		#endregion

		#region エディタとか

		/// <summary>
		/// ...
		/// </summary>
		private void button2_Click(object sender, EventArgs e) {
			if (openFileDialog1.ShowDialog(this) == DialogResult.OK) {
				textBox1.Text = openFileDialog1.FileName;
			}
		}

		#endregion
	}
}